/*
 *     file: RegexSortingComparator.java
 *  package: org.eclipse.regex.tools.util
 *
 * This program and the accompanying materials are made available under 
 * the terms of the Common Public License v1.0 which accompanies this 
 * distribution, and is available at the following URL: 
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Copyright(c) 2003-2005 by the authors indicated in the @author tags.
 * All Rights are Reserved by the various authors.
 *
 ########## DO NOT EDIT ABOVE THIS LINE ########## */

package org.eclipse.regex.tools.util;

import java.text.Collator;
import java.util.Comparator;
import java.util.regex.Pattern;

/**
 * A comparator that compares instances of <code>java.util.regex.Pattern</code> based
 * upon the string used to compile the pattern and the flags.
 * 
 * @author <a href=mailto:kraythe@arcor.de>Robert Simmons Jr. (Kraythe) </a>
 * @see java.util.regex.Pattern
 */
public final class RegexSortingComparator implements Comparator {

    /**
     * Compares the patterns returning them in locale specific alphabetic order.
     * <p>
     * If two patterns are alphabetically identical, the flags of the pattern are used to
     * try to further differentiate the two patterns.
     * </p>
     * 
     * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
     * @throws NullPointerException If either parameter is <code>null</code>.
     * @throws ClassCastException If either parameter is not an instance of
     *             <code>java.util.regex.Pattern</code>.
     */
    public int compare(final Object first, final Object second) {
        // -- Check parameters and set up.
        if (first == null) {
            throw new NullPointerException("first"); //$NON-NLS-1$
        }
        if (second == null) {
            throw new NullPointerException("second"); //$NON-NLS-1$
        }
        final Pattern firstPattern = (Pattern)first;
        final Pattern secondPattern = (Pattern)second;

        // -- Do the comparison based on the string pattern used to compile the pattern.
        // uses the java.text.Collator class to do the work. If two patterns are identical
        // according to the Collator, then compare them based on their flags.
        final Collator localCollator = Collator.getInstance();
        final int result = localCollator.compare(firstPattern.pattern(), secondPattern.pattern());
        if (result == 0) {
            return firstPattern.flags() - secondPattern.flags();
        }
        return result;
    }
}

/* ########## End of File ########## */