/*
 *     file: ISLeak.java
 *  package: org.eclipse.swt.sleak.core
 *
 * This program and the accompanying materials are made available under 
 * the terms of the Common Public License v1.0 which accompanies this 
 * distribution, and is available at the following URL: 
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Copyright(c) 2003-2005 by the authors indicated in the @author tags.
 * All Rights are Reserved by the various authors.
 *
 ########## DO NOT EDIT ABOVE THIS LINE ########## */

package org.eclipse.swt.sleak.core;

import java.util.SortedSet;
import org.eclipse.swt.widgets.Display;

/**
 * Declares the S-Leak cache construction mechanisms and provides access to the collected
 * data.
 * 
 * @author <a href=mailto:kraythe@arcor.de>Robert Simmons Jr. (Kraythe) </a>
 */
public interface ISLeak {

    /**
     * Add a listener to S-Leak core events. <br>
     * <b>WARNING: listeners are stored in <code>WeakReference</code> s!! </b> <br>
     * Currently <code>SLeakEvent</code> s are only fired when a new cache is added or
     * caches are removed.
     * 
     * @see java.lang.ref.WeakReference
     * @param listener The listener to add.
     */
    public void addSLeakListener(final ISLeakEventListener listener);

    /**
     * Explicitly remove a listener to property S-Leak core events. <br>
     * Note that it is not necessary that you remove listeners since they are held in
     * <code>WeakReference</code> s anyway.
     * 
     * @see java.lang.ref.WeakReference
     * @param listener The listener to remove.
     */
    public void removeSLeakListener(final ISLeakEventListener listener);

    /**
     * Getter for caches: The caches that have been created.
     * 
     * @return Returns the caches.
     */
    public SortedSet getCaches();

    /**
     * Takes a snapshot of the current device info.
     * 
     * @return The index of the new snapshot.
     */
    public ISnapshotDeviceDataCache takeSnapshot();

    /**
     * Creates a delta between first and second caches.
     * 
     * @param first The first cache to compare.
     * @param second The second cache to compare.
     * @return The index in in the caches for the delta.
     */
    public IDeltaDeviceDataCache createDelta(final IDeviceDataCache first,
                                             final IDeviceDataCache second);

    /**
     * Getter for display: The display that this SLeakOld is working on.
     * 
     * @return Returns the display.
     */
    public Display getDisplay();

    /**
     * Clears all of the data caches.
     */
    public void clearCaches();

    /**
     * Clears the given cache.
     */
    public void clearCache(final IDeviceDataCache cache);

    /**
     * Starts monitoring by creating a temporary snapshot and tracks until
     * <code>stopTracker()</code> is called. The tracking methods allow a user to build
     * a snapshot over an arbitrary period of time and see the effect on resources.
     * <p>
     * The performance minded developer should note that this method doesnt "monitor"
     * per-se, but creates a snapshot which is compared to the stop monitor snapshot to
     * create the delta.
     * </p>
     */
    public void startMonitor();

    /**
     * Check to see if sleak is monitoring.
     * 
     * @return a value of <code>true</code> if monitoring and <code>false</code>
     *         otherwise.
     */
    public boolean isMonitoring();

    /**
     * Stops monitoring and calculates the resulting delta cache.
     * 
     * @return The index of the cache.
     */
    public IDeltaDeviceDataCache stopMonitor();

    /**
     * Gets the number of caches currently being managed.
     * 
     * @return The number of caches being managed.
     */
    public int getCacheCount();

    /**
     * Gets whether there are any caches stored.
     * 
     * @return A value of <code>true</code> if there are no caches and a value of
     *         <code>false</code> otherwise.
     */
    public boolean isEmpty();

    /**
     * Check to see if the display that this <code>ISLeak</code> is monitoring is
     * tracking device data.
     * 
     * @return Whether or not the display is tracking.
     */
    public boolean isDisplayTracking();
}

/* ########## End of File ########## */