/*
 *     file: PreviewPanel.java
 *  package: org.eclipse.swt.sleak.plugin.views
 *
 * This program and the accompanying materials are made available under 
 * the terms of the Common Public License v1.0 which accompanies this 
 * distribution, and is available at the following URL: 
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Copyright(c) 2003-2005 by the authors indicated in the @author tags.
 * All Rights are Reserved by the various authors.
 *
 ########## DO NOT EDIT ABOVE THIS LINE ########## */

package org.eclipse.swt.sleak.plugin.views;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Region;
import org.eclipse.swt.graphics.TextLayout;
import org.eclipse.swt.sleak.core.IObjectDeviceData;
import org.eclipse.swt.sleak.core.ObjectCategory;
import org.eclipse.swt.sleak.plugin.SLeakPlugin;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

/**
 * A composite that shows a preview of a selected resource.
 * 
 * @author <a href=mailto:kraythe@arcor.de>Robert Simmons Jr. (Kraythe) </a>
 */
class PreviewPanel extends Composite {

    /** A cached image. */
    private final Image imgColor = SLeakPlugin.getPluginImage("icons/color.gif").createImage(); //$NON-NLS-1$

    /** A label to be used in showing text. */
    private final Label lblText;

    /** A label to be used in showing fonts. */
    private final Label lblFont;

    /** A label to be used in showing color. */
    private final Label lblColor;

    /** A label to be used in showing images. */
    private final Label lblImage;

    /** A label to be used in showing a blank field. */
    private final Label lblTextLayout;

    /** A label to be used in showing a blank field. */
    private final Label lblBlank;

    /** The layout keeping track of the components. */
    private final StackLayout layout;

    /**
     * Creates a new <code>PreviewPanel</code> with the given parameters.
     * 
     * @param parent The parent composite.
     */
    PreviewPanel(final Composite parent) {
        super(parent, SWT.NONE);

        // This composite is implemented with several labels, one for each type of object
        // that can be previewed. These labels are switched out as the top contol in
        // the stack layout. There is one extra label for when no object is being
        // previewed.
        this.layout = new StackLayout();
        this.setLayout(this.layout);
        this.lblText = new Label(this, SWT.NONE);
        this.lblColor = new Label(this, SWT.NONE);
        this.lblImage = new Label(this, SWT.NONE);
        this.lblImage.setImage(this.imgColor);
        this.lblFont = new Label(this, SWT.NONE);
        this.lblTextLayout = new Label(this, SWT.NONE);
        this.lblBlank = new Label(this, SWT.NONE);
    }

    /**
     * Set the preview item to the given item and show the preview for that item.
     * 
     * @param item The item to preview or null if no item is to be previewed.
     */
    public void setPreviewItem(final IObjectDeviceData item) {
        // Switch the appropriate panel to the top and calculate the preview of
        // the item to be shown.
        if (null == item) {
            this.layout.topControl = this.lblBlank;
            layout();
            return;
        }
        final ObjectCategory category = item.getCategory();
        if (category == ObjectCategory.COLOR) {
            final Color color = (Color)item.getObject();
            this.lblColor.setBackground(color);
            this.layout.topControl = this.lblColor;
            layout();
        } else if (category == ObjectCategory.CURSOR) {
            final Cursor cursor = (Cursor)item.getObject();
            this.lblText.setText(cursor.toString());
            this.layout.topControl = this.lblText;
            layout();
        } else if (category == ObjectCategory.FONT) {
            final Font font = (Font)item.getObject();
            final FontData[] fontData = font.getFontData();
            final StringBuffer buf = new StringBuffer();
            for (int i = 0; i < fontData.length; i++) {
                final FontData element = fontData[i];
                buf.append(element.getName()).append(" ") //$NON-NLS-1$
                   .append(element.getHeight()).append(" "); //$NON-NLS-1$
                String style = "NORMAL"; //$NON-NLS-1$
                int bits = element.getStyle();
                if (bits != 0) {
                    if ((bits & SWT.BOLD) != 0)
                        style = "BOLD "; //$NON-NLS-1$
                    if ((bits & SWT.ITALIC) != 0)
                        style += "ITALIC"; //$NON-NLS-1$
                }
                buf.append(style).append("\n"); //$NON-NLS-1$
            }
            this.lblFont.setText(buf.toString());
            this.lblFont.setFont(font);
            this.layout.topControl = this.lblFont;
            layout();
        } else if (category == ObjectCategory.GC) {
            final GC gc = (GC)item.getObject();
            this.lblText.setText(gc.toString());
            this.layout.topControl = this.lblText;
            layout();
        } else if (category == ObjectCategory.IMAGE) {
            final Image image = (Image)item.getObject();
            if (!image.isDisposed()) {
                this.lblImage.setImage(image);
                this.layout.topControl = this.lblImage;
            } else {
                this.lblText.setText(image.toString());
                this.layout.topControl = this.lblText;
            }
            layout();
        } else if (category == ObjectCategory.REGION) {
            final Region region = (Region)item.getObject();
            this.lblText.setText(region.toString());
            this.layout.topControl = this.lblText;
            layout();
        } else if (category == ObjectCategory.TEXT_LAYOUT) {
            final TextLayout textLayout = (TextLayout)item.getObject();
            this.lblTextLayout.setText(textLayout.toString());
            this.layout.topControl = this.lblTextLayout;
            layout();
        } else {
            return;
        }
    }

    /**
     * @see org.eclipse.swt.widgets.Widget#dispose()
     */
    public void dispose() {
        this.imgColor.dispose();
        super.dispose();
    }

}

/* ########## End of File ########## */