/*
 *     file: SLeakView.java
 *  package: org.eclipse.swt.sleak.plugin.views
 *
 * This program and the accompanying materials are made available under 
 * the terms of the Common Public License v1.0 which accompanies this 
 * distribution, and is available at the following URL: 
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Copyright(c) 2003-2005 by the authors indicated in the @author tags.
 * All Rights are Reserved by the various authors.
 *
 ########## DO NOT EDIT ABOVE THIS LINE ########## */

package org.eclipse.swt.sleak.plugin.views;

import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.sleak.core.IObjectDeviceData;
import org.eclipse.swt.sleak.core.ISLeak;
import org.eclipse.swt.sleak.core.SLeakFactory;
import org.eclipse.swt.sleak.plugin.SLeakPlugin;
import org.eclipse.swt.sleak.plugin.actions.RemoveAllCaches;
import org.eclipse.swt.sleak.plugin.actions.RemoveCache;
import org.eclipse.swt.sleak.plugin.actions.Snapshot;
import org.eclipse.swt.sleak.plugin.actions.StartMonitor;
import org.eclipse.swt.sleak.plugin.actions.StopMonitor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.part.ViewPart;

/**
 * A view that shows the various caches managed by an ISLeak instance and allows the use
 * to examine the cache in detail.
 * <p>
 * In addition to a list of all caches, the view provides facilites to collect and remove
 * caches. For the objects in the caches, the user can see a preview (if appropriate and
 * examine the stack trace of the resource at creation time.
 * <p>
 * 
 * @author <a href=mailto:kraythe@arcor.de>Robert Simmons Jr. (Kraythe) </a>
 * @see org.eclipse.swt.sleak.core.ISLeak
 */
public class SLeakView extends ViewPart implements ISelectionChangedListener {

    /** The parent of the view composite. */
    private Composite parent = null;

    /** Contains the content of the view. */
    private Composite content = null;

    /** The tree viewer showing the caches. */
    private TreeViewer trvCaches = null;

    /** The tab folder containing the resource information. */
    private TabFolder tfldrInfo = null;

    /** The label providing a preview of the resource. */
    private PreviewPanel preview = null;

    /** The list containing the stack trace. */
    private ListViewer lstvTrace = null;

    /** The <code>ViewerFilter</code> used for filtering the stack trace. */
    private StackTraceFilter filterTrace = null;

    /** The sleak instance to use. */
    private ISLeak sleak = null;

    /**
     * Creates a new <code>SLeakView</code> with the given parameters.
     */
    public SLeakView() {
        // nothing needed.
    }

    /**
     * This is a callback that will allow us to create the viewer and initialize it.
     * 
     * @param parent The parent composite.
     */
    public void createPartControl(final Composite parent) {
        this.parent = parent;
        this.sleak = SLeakFactory.getISLeakInstance(parent.getDisplay());
        constructView();
        configureActions();
    }

    /**
     * Configure's the view's action bars.
     */
    protected void configureActions() {
        IActionBars bars = getViewSite().getActionBars();
        IToolBarManager manager = bars.getToolBarManager();
        manager.add(new Snapshot(this.sleak));
        manager.add(new StartMonitor(this.sleak));
        manager.add(new StopMonitor(this.sleak));
        manager.add(new RemoveCache(this.sleak, this.trvCaches));
        manager.add(new RemoveAllCaches(this.sleak));
    }

    /**
     * Construct the SWT and JFace widgets for the view.
     * 
     * @param parent The parent composite to put everything in.
     */
    private void constructView() {
        this.content = new Composite(this.parent, SWT.NONE);
        {
            this.content.setLayout(new GridLayout(1, false));
        }

        // -- The sash in the middle dividing everything.
        // The left side will contain this.trvCaches and the
        // right side will contain this.tfldrInfo
        final SashForm sashForm;
        {
            sashForm = new SashForm(this.content, SWT.HORIZONTAL);
            final GridData gdata = new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.FILL_BOTH);
            sashForm.setLayoutData(gdata);
        }

        {// -- The tree of stored caches.
            this.trvCaches = new TreeViewer(sashForm, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL
                                                      | SWT.BORDER);
            final GridData gdata = new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.FILL_BOTH);
            gdata.widthHint = 300;
            this.trvCaches.getTree().setLayoutData(gdata);
            this.trvCaches.getTree().setToolTipText(I18N.TOOLTIP_CACHE_TREE);
            this.trvCaches.setLabelProvider(new CacheLabelProvider());
            this.trvCaches.setContentProvider(new CacheContentProvider());
            this.trvCaches.setInput(this.sleak);
            this.trvCaches.addSelectionChangedListener(this);
        }

        { // -- The tab folder containg resource information.
            this.tfldrInfo = new TabFolder(sashForm, SWT.TOP);
            final GridData gdata = new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.FILL_BOTH);
            this.tfldrInfo.setLayoutData(gdata);
        }

        { // -- The label used in the preview tab page.
            this.preview = new PreviewPanel(this.tfldrInfo);
            this.preview.setToolTipText(I18N.LBL_PREVIEW);
        }

        { // -- The list used to display stack trace info.
            this.lstvTrace = new ListViewer(this.tfldrInfo, SWT.H_SCROLL | SWT.V_SCROLL
                                                            | SWT.BORDER);
            this.lstvTrace.setLabelProvider(new LabelProvider());
            this.lstvTrace.setContentProvider(new StackTraceContentProvider());
            this.filterTrace = new StackTraceFilter(this.lstvTrace);
            SLeakPlugin.getDefault()
                       .getPreferenceStore()
                       .addPropertyChangeListener(this.filterTrace);
            this.lstvTrace.addFilter(this.filterTrace);
            this.lstvTrace.getList().setToolTipText(I18N.TOOLTIP_TRACE);
        }

        { // -- The page in the tab folder for a preview.
            final TabItem tabiPreview = new TabItem(this.tfldrInfo, SWT.NONE, 0);
            tabiPreview.setToolTipText(I18N.TOOLTIP_PREVIEW);
            tabiPreview.setImage(null);
            tabiPreview.setText(I18N.LBL_PREVIEW);
            tabiPreview.setControl(this.preview);
        }

        { // -- The page in the tab folder for a stack trace.
            final TabItem tabiTrace = new TabItem(this.tfldrInfo, SWT.NONE, 0);

            tabiTrace.setToolTipText(I18N.TOOLTIP_TRACE);
            tabiTrace.setImage(null);
            tabiTrace.setText(I18N.LBL_TRACE);
            tabiTrace.setControl(this.lstvTrace.getList());
        }
    }

    /**
     * Passing the focus request to the viewer's control.
     */
    public void setFocus() {
        // no code needed now.
    }

    /**
     * @see org.eclipse.ui.IWorkbenchPart#dispose()
     */
    public void dispose() {
        this.trvCaches.removeSelectionChangedListener(this);
        super.dispose();
    }

    /**
     * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
     */
    public void selectionChanged(final SelectionChangedEvent event) {
        if (event.getSource() == this.trvCaches) {
            // -- Handle selection on the tree by setting the preview and stack trace
            // for the selected item.
            final IStructuredSelection selection = (IStructuredSelection)this.trvCaches.getSelection();
            if (selection.isEmpty()) {
                this.preview.setPreviewItem(null);
                this.lstvTrace.setInput(null);
                return;
            }
            if (selection.size() == 1) {
                if (selection.getFirstElement() instanceof IObjectDeviceData) {
                    final IObjectDeviceData node = (IObjectDeviceData)selection.getFirstElement();
                    this.preview.setPreviewItem(node);
                    this.lstvTrace.setInput(node.getStackTrace());
                } else {
                    this.preview.setPreviewItem(null);
                    this.lstvTrace.setInput(null);
                }
            }
        }
    }
}

/* ########## End of File ########## */