/*
 *     file: RegexPatternSetFieldEditor.java
 *  package: org.eclipse.regex.tools.ui
 *
 * This program and the accompanying materials are made available under 
 * the terms of the Common Public License v1.0 which accompanies this 
 * distribution, and is available at the following URL: 
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Copyright(c) 2003-2005 by the authors indicated in the @author tags.
 * All Rights are Reserved by the various authors.
 *
 ########## DO NOT EDIT ABOVE THIS LINE ########## */

package org.eclipse.regex.tools.ui;

import java.util.Collection;
import java.util.Iterator;
import java.util.SortedSet;
import java.util.StringTokenizer;
import java.util.TreeSet;
import java.util.regex.Pattern;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.regex.tools.util.RegexSortingComparator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

/**
 * A field editor for editing a sorted set of regular expression patterns.
 * 
 * @author <a href=mailto:kraythe@arcor.de>Robert Simmons Jr. (Kraythe) </a>
 */
public class RegexPatternSetFieldEditor extends FieldEditor implements SelectionListener,
                                                           IDoubleClickListener {

    /** A character used to separate the pattern from the flags in the encoded regexp. */
    private final static char FLAG_SEP = (char)1;

    /** A character used to separate the patterns from each other when encoded. */
    private final static char PATTERN_SEP = (char)2;

    /** The encoding delimiters string; the concatenation of the delimiter chars. */
    private final static String DELIMITERS = Character.toString(FLAG_SEP)
                                             + Character.toString(PATTERN_SEP);

    /** Holds the list viewer for the list of patterns. */
    private TableViewer tbvPatternList;

    /** The regex pattern editor. */
    private RegexPatternEditor rpePattern;

    /** The label for the editor. */
    private Label lblTitle;

    /** The button for adding patterns. */
    private Button btnAddPattern;

    /** The button for remiving patterns. */
    private Button btnRemovePattern;

    /** The sorted set of patterns. */
    private final SortedSet patterns;

    /** The composite that holds all the contrls. */
    private Composite composite;

    /**
     * Creates a new <code>RegexPatternSetFieldEditor</code> with the given parameters.
     * 
     * @param preferenceName The name of the preference to edit.
     * @param labelText The label text to use for the editor; if null is used then no text
     *            will be displayed on the border.
     * @param parent The parent composite of the editor.
     */
    public RegexPatternSetFieldEditor(final String preferenceName, final String labelText,
                                      final Composite parent) {
        super(preferenceName, labelText, parent);
        this.patterns = new TreeSet(new RegexSortingComparator());
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#adjustForNumColumns(int)
     */
    protected void adjustForNumColumns(final int numColumns) {
        ((GridData)this.composite.getLayoutData()).horizontalSpan = numColumns;
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#doFillIntoGrid(org.eclipse.swt.widgets.Composite,
     *      int)
     */
    protected void doFillIntoGrid(final Composite parent, final int numColumns) {
        final Group group = new Group(parent, SWT.NONE);
        { // Initialize the group
            group.setLayout(new GridLayout(1, true));
            group.setText(getLabelText());
            final GridData gridData = new GridData(GridData.FILL_HORIZONTAL
                                                   | GridData.GRAB_HORIZONTAL);
            gridData.horizontalSpan = numColumns;
            group.setLayoutData(gridData);
        }

        { // the containign composite
            this.composite = new Composite(group, SWT.NONE);
            this.composite.setLayout(new GridLayout(2, false));
            final GridData gridData = new GridData(GridData.FILL_HORIZONTAL
                                                   | GridData.GRAB_HORIZONTAL);
            this.composite.setLayoutData(gridData);
        }

        { // The list of patterns.
            final Table table = new Table(this.composite, SWT.NONE | SWT.SINGLE
                                                          | SWT.FULL_SELECTION);
            table.setToolTipText(I18N.TOOLTIP_PATTERN_TABLE);
            table.setHeaderVisible(true);
            { // create columns.
                // -- Flags column.
                final TableColumn colFlags = new TableColumn(table, SWT.LEFT, 0);
                colFlags.setText(I18N.COL_FLAGS);
                colFlags.setResizable(false);
                final GC gc = new GC(table);
                final Point extent;
                try {
                    extent = gc.stringExtent("XXXXXXXXX"); //$NON-NLS-1$
                    colFlags.setWidth(extent.x);
                } finally {
                    gc.dispose();
                }
                // -- Patterns column.
                final TableColumn colPatterns = new TableColumn(table, SWT.LEFT, 1);
                colPatterns.setText(I18N.COL_PATTERNS);
                colPatterns.setResizable(true);
                colPatterns.setWidth(extent.x * 3);
                table.pack();
            }

            this.tbvPatternList = new TableViewer(table);
            table.addControlListener(new TableControlAdapter());
            this.tbvPatternList.setLabelProvider(new RegexPatternLabelProvider());
            this.tbvPatternList.setContentProvider(new RegexPatternsContentProvider());
            this.tbvPatternList.addDoubleClickListener(this);
            final GridData gridData = new GridData(GridData.FILL_BOTH);
            this.tbvPatternList.getTable().setLayoutData(gridData);
            this.tbvPatternList.getTable().setLinesVisible(true);
        }

        { // The pattern editor.
            this.rpePattern = new RegexPatternEditor(this.composite, SWT.NONE);
            final GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
            this.rpePattern.setLayoutData(gridData);
        }

        { // The pattern remove button
            this.btnRemovePattern = new Button(this.composite, SWT.PUSH);
            this.btnRemovePattern.setText(I18N.BTN_REMOVE_PATTERN);
            this.btnRemovePattern.addSelectionListener(this);
            final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
            this.btnRemovePattern.setLayoutData(gridData);
        }

        { // The add pattern button
            this.btnAddPattern = new Button(this.composite, SWT.PUSH);
            this.btnAddPattern.setText(I18N.BTN_ADD_PATTERN);
            this.btnAddPattern.addSelectionListener(this);
            final GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
            this.btnAddPattern.setLayoutData(gridData);
        }
    }

    /**
     * Encode the given pattern collection into the right value for storage. This method
     * is public so that it may be used by plugins to set up defaults.
     * 
     * @param patterns The patterns to encode.
     * @return The encoded patterns
     */
    public static String encodePatterns(final Collection patterns) {
        // See inlined documentation in decodePatterns() for encoding information.
        final StringBuffer encodeBuffer = new StringBuffer(64 * patterns.size());
        for (final Iterator iter = patterns.iterator(); iter.hasNext();) {
            final Pattern pattern = (Pattern)iter.next();
            encodeBuffer.append(pattern.flags());
            encodeBuffer.append(FLAG_SEP);
            encodeBuffer.append(pattern);
            encodeBuffer.append(PATTERN_SEP);
        }
        return encodeBuffer.toString();
    }

    /**
     * Decode the given patterns from the encoded string. This method is public so that it
     * may be used by plugins to set up defaults.
     * 
     * @param encoded The encoded patterns.
     * @return The sorted set of decoded patterns.
     */
    public static SortedSet decodePatterns(final String encoded) {
        // Encoding Info: The patterns are encoded a long concatenated list of in
        // pairs of strings separated by the PATTERN_SEP character. The first token in
        // each pair will be the flags and the second will be the pattern itself.
        final SortedSet patterns = new TreeSet(new RegexSortingComparator());
        final StringTokenizer tok = new StringTokenizer(encoded, DELIMITERS, false);
        while (tok.hasMoreTokens()) {
            final int flags = Integer.parseInt(tok.nextToken());
            final String patternString = tok.nextToken();
            final Pattern pattern = Pattern.compile(patternString, flags);
            patterns.add(pattern);
        }
        return patterns;
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#doLoad()
     */
    protected void doLoad() {
        final String encoded = getPreferenceStore().getString(getPreferenceName());
        this.patterns.addAll(decodePatterns(encoded));
        this.tbvPatternList.setInput(this.patterns);
        this.tbvPatternList.refresh();
    }

    /**
     * Persists the patterns in the list only; ignores the pattern editor.
     * 
     * @see org.eclipse.jface.preference.FieldEditor#doStore()
     */
    protected void doStore() {
        final String encoded = RegexPatternSetFieldEditor.encodePatterns(this.patterns);
        getPreferenceStore().setValue(this.getPreferenceName(), encoded);
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#doLoadDefault()
     */
    protected void doLoadDefault() {
        // Essentially clears the list of patterns.
        this.patterns.clear();
        this.tbvPatternList.setInput(this.patterns);
        this.tbvPatternList.refresh();
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#getNumberOfControls()
     */
    public int getNumberOfControls() {
        return 1;
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#isValid()
     */
    public boolean isValid() {
        return super.isValid();
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#setEnabled(boolean,
     *      org.eclipse.swt.widgets.Composite)
     */
    public void setEnabled(final boolean enabled, final Composite parent) {
        this.tbvPatternList.getControl().setEnabled(enabled);
        this.rpePattern.setEnabled(enabled);
        this.btnAddPattern.setEnabled(enabled);
        this.btnRemovePattern.setEnabled(enabled);
        super.setEnabled(enabled, parent);
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#setFocus()
     */
    public void setFocus() {
        this.rpePattern.setFocus();
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#getLabelControl()
     */
    protected Label getLabelControl() {
        return this.lblTitle;
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#setLabelText(java.lang.String)
     */
    public void setLabelText(final String text) {
        this.lblTitle.setText(text);
    }

    /**
     * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
     */
    public void widgetDefaultSelected(final SelectionEvent event) {
        // ignore
    }

    /**
     * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
     */
    public void widgetSelected(final SelectionEvent event) {
        final Object source = event.getSource();
        if (source == this.btnAddPattern) {
            final Pattern pattern = this.rpePattern.getRegexPattern();
            if (null != pattern) {
                this.patterns.add(pattern);
                this.tbvPatternList.refresh();
            }
        } else if (source == this.btnRemovePattern) {
            final Pattern pattern = getCurrentlySelectedPattern();
            if (null != pattern) {
                this.patterns.remove(pattern);
                this.tbvPatternList.refresh();
            }
        } else {
            throw new AssertionError(I18N.ERR_UNEXPECTED_BRANCH);
        }

    }

    /**
     * Helper method for getting the currently selected pattern in the list.
     */
    private final Pattern getCurrentlySelectedPattern() {
        final IStructuredSelection sel = (IStructuredSelection)this.tbvPatternList.getSelection();
        if (sel.isEmpty()) {
            return null;
        }
        return (Pattern)sel.getFirstElement();
    }

    /**
     * @see org.eclipse.jface.viewers.IDoubleClickListener#doubleClick(org.eclipse.jface.viewers.DoubleClickEvent)
     */
    public void doubleClick(final DoubleClickEvent event) {
        final Object source = event.getSource();
        if (this.tbvPatternList == source) {
            final Pattern pattern = getCurrentlySelectedPattern();
            if (pattern != null) {
                this.rpePattern.setRegexPattern(pattern);
            }
        }
        throw new AssertionError(I18N.ERR_UNEXPECTED_BRANCH);
    }

    private final class TableControlAdapter extends ControlAdapter {

        /**
         * @see org.eclipse.swt.events.ControlListener#controlResized(org.eclipse.swt.events.ControlEvent)
         */
        public void controlResized(ControlEvent e) {
            final Table table = RegexPatternSetFieldEditor.this.tbvPatternList.getTable();
            final int colFlagsWidth = table.getColumn(0).getWidth();
            final int clientAreaWidth = table.getClientArea().width;
            final int borderWidth = table.getBorderWidth();

            // Calculate the resulting pattern column width.
            final int colPatternWidth = clientAreaWidth - (2 * borderWidth) - colFlagsWidth;
            table.getColumn(1).setWidth(colPatternWidth);
        }
    }
}

/* ########## End of File ########## */