/*
 *     file: SLeak.java
 *  package: org.eclipse.swt.sleak.core
 *
 * This program and the accompanying materials are made available under 
 * the terms of the Common Public License v1.0 which accompanies this 
 * distribution, and is available at the following URL: 
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Copyright(c) 2003-2005 by the authors indicated in the @author tags.
 * All Rights are Reserved by the various authors.
 *
 ########## DO NOT EDIT ABOVE THIS LINE ########## */

package org.eclipse.swt.sleak.core;

import java.util.Iterator;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.WeakHashMap;
import org.eclipse.swt.widgets.Display;

/**
 * Default implementation of the <code>ISLeak</code> interface.
 * 
 * @author <a href=mailto:kraythe@arcor.de>Robert Simmons Jr. (Kraythe) </a>
 * @see org.eclipse.swt.sleak.core.ISLeak
 */
class SLeak implements ISLeak {

    /** Snapshot cache used for monitoring. */
    private SnapshotDeviceDataCache monitorCache = null;

    /** The display that this SLeakOld is working on. */
    final Display display;

    /** The caches that have been created. */
    private final SortedSet caches = new TreeSet();

    /** Listener storage. */
    private final WeakHashMap listeners = new WeakHashMap();

    /**
     * @see org.eclipse.swt.sleak.core.ISLeak#addSLeakListener(org.eclipse.swt.sleak.core.ISLeakEventListener)
     */
    public void addSLeakListener(final ISLeakEventListener listener) {
        this.listeners.put(listener, null);
    }

    /**
     * @see org.eclipse.swt.sleak.core.ISLeak#removeSLeakListener(org.eclipse.swt.sleak.core.ISLeakEventListener)
     */
    public void removeSLeakListener(final ISLeakEventListener listener) {
        this.listeners.remove(listener);
    }

    /**
     * Fire a SLeakEvent caches updated event to the listeners.
     * 
     * @param event The event to fire.
     */
    protected void fireSLeakCacheEvent(final SLeakEvent event) {
        for (final Iterator iter = this.listeners.keySet().iterator(); iter.hasNext();) {
            final ISLeakEventListener listener = (ISLeakEventListener)iter.next();
            listener.doCachesUpdated(event);
        }
    }

    /**
     * Fire a SLeakEvent monitor state event to the listeners.
     * 
     * @param event The event to fire.
     */
    protected void fireSLeakMonitorEvent(final SLeakEvent event) {
        for (final Iterator iter = this.listeners.keySet().iterator(); iter.hasNext();) {
            final ISLeakEventListener listener = (ISLeakEventListener)iter.next();
            listener.doMonitorStateChanged(event);
        }
    }

    /**
     * Creates a new <code>SLeak</code> with the given parameters.
     * 
     * @param display The display to work on.
     */
    public SLeak(final Display display) {
        this.display = display;
    }

    /**
     * @see org.eclipse.swt.sleak.core.ISLeak#getCaches()
     */
    public SortedSet getCaches() {
        return this.caches;
    }

    /**
     * @see org.eclipse.swt.sleak.core.ISLeak#takeSnapshot()
     */
    public ISnapshotDeviceDataCache takeSnapshot() {
        final ISnapshotDeviceDataCache snap = new SnapshotDeviceDataCache(this.display);
        this.caches.add(snap);
        fireSLeakCacheEvent(new SLeakEvent(this));
        return snap;
    }

    /**
     * @see org.eclipse.swt.sleak.core.ISLeak#createDelta(int, int)
     */
    public IDeltaDeviceDataCache createDelta(final IDeviceDataCache first,
                                             final IDeviceDataCache second) {
        final IDeltaDeviceDataCache deltaOne = new DeltaDeviceDataCache(first, second);
        this.caches.add(deltaOne);
        fireSLeakCacheEvent(new SLeakEvent(this));
        return deltaOne;
    }

    /**
     * @see org.eclipse.swt.sleak.core.ISLeak#getDisplay()
     */
    public Display getDisplay() {
        return this.display;
    }

    /**
     * @see org.eclipse.swt.sleak.core.ISLeak#clearCaches()
     */
    public void clearCaches() {
        this.caches.clear();
        fireSLeakCacheEvent(new SLeakEvent(this));
    }

    /**
     * @see org.eclipse.swt.sleak.core.ISLeak#clearCache(org.eclipse.swt.sleak.core.IDeltaDeviceDataCache)
     */
    public void clearCache(final IDeviceDataCache cache) {
        this.caches.remove(cache);
        fireSLeakCacheEvent(new SLeakEvent(this));
    }

    /**
     * @see org.eclipse.swt.sleak.core.ISLeak#startMonitor()
     */
    public void startMonitor() {
        if (this.monitorCache != null) {
            throw new IllegalStateException(I18N.ERR_ALREADY_MONITORING);
        }
        this.monitorCache = new SnapshotDeviceDataCache(this.display);
        fireSLeakMonitorEvent(new SLeakEvent(this));
    }

    /**
     * @see org.eclipse.swt.sleak.core.ISLeak#isMonitoring()
     */
    public boolean isMonitoring() {
        return this.monitorCache != null;
    }

    /**
     * @see org.eclipse.swt.sleak.core.ISLeak#stopMonitor()
     */
    public IDeltaDeviceDataCache stopMonitor() {
        final SnapshotDeviceDataCache endMonitorCache = new SnapshotDeviceDataCache(this.display);
        final IDeltaDeviceDataCache delta = new DeltaDeviceDataCache(this.monitorCache,
                                                                     endMonitorCache);
        this.caches.add(delta);
        this.monitorCache = null;
        fireSLeakCacheEvent(new SLeakEvent(this));
        fireSLeakMonitorEvent(new SLeakEvent(this));
        return delta;
    }

    /**
     * @see org.eclipse.swt.sleak.core.ISLeak#getCacheCount()
     */
    public int getCacheCount() {
        return this.caches.size();
    }

    /**
     * @see org.eclipse.swt.sleak.core.ISLeak#isEmpty()
     */
    public boolean isEmpty() {
        return this.caches.isEmpty();
    }

    /**
     * @see org.eclipse.swt.sleak.core.ISLeak#isDisplayTracking()
     */
    public boolean isDisplayTracking() {
        return this.display.getDeviceData().tracking;
    }
}

/* ########## End of File ########## */