/*
 *     file: CacheLabelProvider.java
 *  package: org.eclipse.swt.sleak.plugin.views
 *
 * This program and the accompanying materials are made available under 
 * the terms of the Common Public License v1.0 which accompanies this 
 * distribution, and is available at the following URL: 
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Copyright(c) 2003-2005 by the authors indicated in the @author tags.
 * All Rights are Reserved by the various authors.
 *
 ########## DO NOT EDIT ABOVE THIS LINE ########## */

package org.eclipse.swt.sleak.plugin.views;

import java.util.Calendar;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.SortedMap;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.sleak.core.IDeltaDeviceDataCache;
import org.eclipse.swt.sleak.core.IDeviceDataCache;
import org.eclipse.swt.sleak.core.IObjectDeviceData;
import org.eclipse.swt.sleak.core.ISnapshotDeviceDataCache;
import org.eclipse.swt.sleak.core.ObjectCategory;
import org.eclipse.swt.sleak.plugin.SLeakPlugin;

/**
 * A label provider for a cache and its contents.
 * 
 * @author <a href=mailto:kraythe@arcor.de>Robert Simmons Jr. (Kraythe) </a>
 */
public class CacheLabelProvider extends LabelProvider {

    /** A cached image for color resources. */
    private final Image imgColor = SLeakPlugin.getPluginImage("icons/color.gif").createImage(); //$NON-NLS-1$

    /** A cached image for cursor resources. */
    private final Image imgCursor = SLeakPlugin.getPluginImage("icons/cursor.gif").createImage(); //$NON-NLS-1$

    /** A cached image for font resources. */
    private final Image imgFont = SLeakPlugin.getPluginImage("icons/font.gif").createImage(); //$NON-NLS-1$

    /** A cached image for GC resources. */
    private final Image imgGC = SLeakPlugin.getPluginImage("icons/gc.gif").createImage(); //$NON-NLS-1$

    /** A cached image for image resources. */
    private final Image imgImage = SLeakPlugin.getPluginImage("icons/image.gif").createImage(); //$NON-NLS-1$

    /** A cached image for color resources. */
    private final Image imgTextLayout = SLeakPlugin.getPluginImage("icons/text_layout.gif").createImage(); //$NON-NLS-1$

    /** A cached image for region resources. */
    private final Image imgRegion = SLeakPlugin.getPluginImage("icons/region.gif").createImage(); //$NON-NLS-1$

    /** A cached image for delta caches. */
    private final Image imgDelta = SLeakPlugin.getPluginImage("icons/delta_cache.gif").createImage(); //$NON-NLS-1$

    /** A cached image for snapshot caches. */
    private final Image imgSnapshot = SLeakPlugin.getPluginImage("icons/snapshot_cache.gif").createImage(); //$NON-NLS-1$

    /** The lookup table used to speed up image recognition. */
    private final Map categoryLookupTable;

    /**
     * Creates a new <code>CacheLabelProvider</code>.
     */
    public CacheLabelProvider() {
        // Build the image lookup table by filing the appropate image by class category.
        final Map tmpLookupTable = new HashMap();
        tmpLookupTable.put(ObjectCategory.COLOR, this.imgColor);
        tmpLookupTable.put(ObjectCategory.CURSOR, this.imgCursor);
        tmpLookupTable.put(ObjectCategory.FONT, this.imgFont);
        tmpLookupTable.put(ObjectCategory.GC, this.imgGC);
        tmpLookupTable.put(ObjectCategory.IMAGE, this.imgImage);
        tmpLookupTable.put(ObjectCategory.REGION, this.imgRegion);
        tmpLookupTable.put(ObjectCategory.TEXT_LAYOUT, this.imgTextLayout);
        this.categoryLookupTable = Collections.unmodifiableMap(tmpLookupTable);
    }

    /**
     * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
     */
    public void dispose() {
        this.imgColor.dispose();
        this.imgCursor.dispose();
        this.imgFont.dispose();
        this.imgGC.dispose();
        this.imgImage.dispose();
        this.imgRegion.dispose();
        this.imgSnapshot.dispose();
        this.imgDelta.dispose();
        this.imgTextLayout.dispose();
        super.dispose();
    }

    /**
     * @see org.eclipse.jface.viewers.ILabelProvider#getImage(java.lang.Object)
     */
    public Image getImage(final Object element) {
        if (element instanceof SortedMap) {
            return null;
        } else if (element instanceof CategoryAndCache) {
            return (Image)this.categoryLookupTable.get(((CategoryAndCache)element).category);
        } else if (element instanceof IDeltaDeviceDataCache) {
            return this.imgDelta;
        } else if (element instanceof ISnapshotDeviceDataCache) {
            return this.imgSnapshot;
        } else if (element instanceof IObjectDeviceData) {
            final ObjectCategory cat = ((IObjectDeviceData)element).getCategory();
            return (Image)this.categoryLookupTable.get(cat);
        }
        throw new AssertionError(I18N.ERR_UNEXPECTED_ELEMENT + element.getClass().getName());
    }

    /**
     * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
     */
    public String getText(final Object element) {
        if (element instanceof SortedMap) {
            return I18N.ROOT_ELEMENT;
        } else if (element instanceof CategoryAndCache) {
            return ((CategoryAndCache)element).category.getTargetClass().getName();
        } else if (element instanceof IDeviceDataCache) {
            final Calendar cal = Calendar.getInstance();
            cal.setTime(((IDeviceDataCache)element).getCreationTime());
            final StringBuffer buf = new StringBuffer(20);
            buf.append(cal.get(Calendar.HOUR_OF_DAY));
            buf.append(":"); //$NON-NLS-1$
            buf.append(cal.get(Calendar.MINUTE));
            buf.append(":"); //$NON-NLS-1$
            buf.append(cal.get(Calendar.SECOND));
            return buf.toString();
        } else if (element instanceof IObjectDeviceData) {
            return "{" + ((IObjectDeviceData)element).getObject().hashCode() //$NON-NLS-1$
                   + "}"; //$NON-NLS-1$
        }
        throw new AssertionError(I18N.ERR_UNEXPECTED_ELEMENT + element.getClass().getName());
    }
}

/* ########## End of File ########## */